<?php
if ( !defined( 'ABSPATH' ) ) exit;

if ( !class_exists( 'fspAdminNotices' ) ) {
/**
 * This class handles displaying notices (ex. expired license) in the admin
 */
class fspAdminNotices {

	/**
	 * Adds the necessary filter and action calls
	 */
	public function __construct() {

		add_action( 'admin_notices', array( $this, 'maybe_add_rtu_expired_license_notice' ) );

		add_action( 'wp_ajax_fspph_hide_rtu_expired_license_notice', array( $this, 'hide_rtu_expired_license_notice' ) );

		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_expired_license_scripts' ) );
	}

	/**
	 * Displays a notice if using RTU with an expired license
	 *
	 */
	public function maybe_add_rtu_expired_license_notice() {
		global $rtb_controller;

		if ( $this->rtu_notice_needed() == false ) { return; }

		?>
	
			<div class='notice notice-error is-dismissible fspph-main-dashboard-expired-license' style='display:none'>
				<div class='fspph-main-dashboard-expired-license-inside'>
					<div class='fspph-license-expired-icon rtb'></div>
					<div class='fspph-main-dashboard-expired-license-text-actions'>
						<div class='fspph-license-expired-text'>
							<p><?php echo sprintf( __( 'The Five-Star Restaurant Reservations Ultimate license on your site is expired. Please <a target=\'_blank\' href=\'%s\'>renew now </a>to ensure that all reservation features continue to work correctly.', 'fsp-premium-helper' ), 'https://www.fivestarplugins.com/license-payment/?Selected=RTU&Quantity=12&License_Key=' . get_option( 'rtb-ultimate-license-key' ) . '&Email=' . $rtb_controller->settings->get_setting( 'ultimate-purchase-email' ) ); ?></p>
						</div>
						<div class='fspph-license-expired-actions'>
							<div class='rtb-license-expired-hide'><?php _e( 'Hide this notice', 'fsp-premium-helper' ); ?></div>
							<?php echo sprintf( __( '<a class=\'fspph-license-expired-action-blue\' target=\'_blank\' href=\'%s\'>Renew Now</a>', 'fsp-premium-helper' ), 'https://www.fivestarplugins.com/license-payment/?Selected=RTU&Quantity=12&License_Key=' . get_option( 'rtb-ultimate-license-key' ) . '&Email=' . $rtb_controller->settings->get_setting( 'ultimate-purchase-email' ) ); ?>
						</div>
					</div>
				</div>
			</div>

		<?php
	}

	public function rtu_notice_needed() {
		global $rtb_controller;
		global $pagenow;

		if ( empty( $rtb_controller ) ) { return false; }

		if ( $pagenow == 'post.php' or $pagenow == 'post-new.php' ) { return false; }

		if ( empty( $rtb_controller->permissions->check_permission( 'api_usage' ) ) ) { return false; }

		if ( get_transient( 'rtb-ultimate-expired-notice-hidden' ) ) { return false; }

		$expiry_information = get_transient( 'rtb-credit-information' );

		if ( empty( $expiry_information ) or ! is_array( $expiry_information ) ) { return false; }

		if ( strtotime( $expiry_information['expiry'] ) >= time() ) { return false; } 

		return true;
	}

	public function hide_rtu_expired_license_notice() {
		global $ewd_upcp_controller;

		// Authenticate request
		if ( ! check_ajax_referer( 'fspph-expired-license-js', 'nonce' ) ) {

			wp_send_json_error(
    			array(
    				'error' => 'badnonce',
    				'msg'   => __( 'The request has been rejected because it does not appear to have come from this site.', 'ultimate-product-catalogue' ),
    			)
    		);
		}
		
		$transient_time = max( DAY_IN_SECONDS * intval( $_POST['hide_notice_time'] ), get_transient( 'rtb-ultimate-expired-notice-hidden' ) );

		set_transient( 'rtb-ultimate-expired-notice-hidden', true, $transient_time );

		die();
	}

	public function enqueue_expired_license_scripts() {

		if ( $this->rtu_notice_needed() == false ) { return; }
		
		wp_enqueue_style( 'fspph-expired-license-css', FSPPH_PLUGIN_URL . '/assets/css/expired-license.css' );
		wp_enqueue_script( 'fspph-expired-license-js', FSPPH_PLUGIN_URL . '/assets/js/expired-license.js', array( 'jquery' ), FSPPH_VERSION, true  );

		wp_localize_script(
			'fspph-expired-license-js',
			'fspph_expired_license',
			array(
				'nonce' => wp_create_nonce( 'fspph-expired-license-js' )
			)
		);
	}
}
} // endif;
